<?php

if (!defined('ABSPATH') || !defined('AUTH_KEY')) {
    exit;
}

class TransacionCloudPaymentGateway extends WC_Payment_Gateway
{
    /**
     * @var TransactionCloudPayload
     */
    private $payloadService;

    /**
     * @var TransactionCloudApi
     */
    private $http;

    public function __construct()
    {
        $this->id = 'transaction_cloud';
        $this->icon = esc_url(plugin_dir_url( __DIR__ ) . '/assets/svg/icon.svg');
        $this->has_fields = false;
        $this->method_title = 'Transaction Cloud';
        $this->method_description = 'Transaction Cloud integration for WooCommerce.';

        $this->init_form_fields();
        $this->init_settings();

        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');

        $this->initHttpClient();
        $this->initPayloadService();

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

        //https://your-domain.com/?wc-api=tc_webhook
        add_action('woocommerce_api_tc_webhook', array($this, 'processWebhook'));
    }

    private function initPayloadService()
    {
        $this->payloadService = new TransactionCloudPayload();
        $this->payloadService->setCiphering('AES-128-CTR');
        $this->payloadService->setKey(AUTH_KEY);
    }

    private function initHttpClient()
    {
        $this->http = new TransactionCloudApi();
        $this->http->addHeader(
            'Authorization: ' . $this->get_option('api_login') . ':' . $this->get_option('api_password')
        );
        $this->http->addHeader(
            'TC-Version: v' . TC_WOO_VERSION
        );

        $env = $this->get_option('environment');

        if ($env == 'production') {
            $this->http->useProductionEnvironment();
        }
    }

    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => 'Enable/Disable',
                'type' => 'checkbox',
                'label' => 'Enable Transaction Cloud',
                'default' => 'yes'
            ),
            'title' => array(
                'title' => 'Title',
                'type' => 'text',
                'description' => 'This controls the title which the user sees during checkout',
                'default' => 'Transaction Cloud',
                'desc_tip' => true,
            ),
            'description' => array(
                'title' => 'Description',
                'type' => 'textarea',
                'description' => 'This controls the description which the user sees during checkout',
                'default' => '',
                'desc_tip' => true,
            ),
            'api_login' => array(
                'title' => 'API Login',
                'type' => 'text',
                'description' => 'Enter your Transacion Cloud API Login',
                'default' => '',
                'desc_tip' => true,
            ),
            'api_password' => array(
                'title' => 'API Password',
                'type' => 'text',
                'description' => 'Enter your Transacion Cloud API password',
                'default' => '',
                'desc_tip' => true
            ),
            'security_tag' => array(
                'title' => 'Security Tag',
                'type' => 'text',
                'description' => 'Transaction Cloud security tag sent as part of webhook request required to verify sender',
                'default' => '',
                'desc_tip' => true
            ),
            'product_id' => array(
                'title' => 'Product ID',
                'type' => 'text',
                'description' => 'Product ID used to initialize transacion',
                'default' => '',
                'desc_tip' => true,
            ),
            'environment' => array(
                'title' => 'Envoronment',
                'type' => 'select',
                'description' => 'Environment',
                'default' => '',
                'desc_tip' => true,
                'options' => [
                    'production' => 'Production',
                    'sandbox' => 'Sandbox'
                ]
            ),
            'webhook_url' => array(
                'title' => 'Webhook URL',
                'type' => 'text',
                'description' => 'Copy this webhook URL and paste it in the Webhooks section of your API settings in your transaction.cloud account. Click <a href="https://app.transaction.cloud/api-set-up" target="_blank">here</a> to access the API settings.',
                'desc_tip' => 'Come to your API settings via <a href="https://app.transaction.cloud/api-set-up" target="_blank">https://app.transaction.cloud/api-set-up</a>, navigate to the Webhooks section, and paste this generated webhook URL in the "Webhook URL" field. Finally, click the "Add new webhook" button.',
                'default' => esc_url( home_url( '/?wc-api=tc_webhook' ) ),
                'custom_attributes' => array(
                'readonly' => 'readonly',
                'disabled' => 'disabled'
                )
            ),
        );
    }

    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        try {
            $payload = $this->payloadService->encode($order_id);
        } catch (TransactionCloudPayloadEncodeException $e) {
            wc_add_notice($e->getMessage(), 'error');
            return false;
        }

        // Prepare the data to send to the Transaction Cloud endpoint.
        $data = array(
            'amount' => $order->get_total(),
            'currency' => $order->get_currency(),
            'payload' => $payload,
        );

        // Send a request to the Transaction Cloud endpoint to get the payment URL.
        try {
            $paymentUrl = $this->getPaymentUrl($data);
        } catch (\Exception $e) {
            $error = sprintf('%s (%s)', $e->getMessage(), $this->get_option('product_id'));
            wc_add_notice($error, 'error');
            return false;
        }

        $paymentUrl = $this->applyUserDetails($paymentUrl, [
            'email' => $order->get_billing_email(),
            'firstname' => $order->get_billing_first_name(),
            'lastname' => $order->get_billing_last_name()
        ]);

        // Reduce stock levels.
        wc_reduce_stock_levels($order_id);

        // Empty cart
        wc_empty_cart();

        // Set the order status to pending.
        $order->update_status('pending', 'Awaiting Transaction Cloud payment.');

        // Add a note to the order.
        $order->add_order_note('Redirecting to Transaction Cloud for payment.');

        // Redirect to the payment URL in a new tab.
        return array(
            'result' => 'success',
            'redirect' => $paymentUrl,
        );
    }

    private function applyUserDetails($url, $data)
    {
        $query = http_build_query(array_filter($data));

        return $url . '?' . $query;
    }

    private function getPaymentUrl($data)
    {
        $productId = $this->get_option('product_id');
        $customizeProduct = new TransactionCloudCustomizeProduct($productId);

        $price = new TransactionCloudPrice();
        $price->setCurrency($data['currency']);
        $price->setValue(floatval($data['amount']));

        $customizeProduct->addPrice($price);
        $customizeProduct->setPayload($data['payload']);

        $customizedProduct = $this->http->customizeProduct($customizeProduct);

        if (!$customizedProduct) {
            return null;
        }

        return $customizedProduct->getRedirectLink();
    }

    public function processWebhook()
    {
        $data = json_decode(file_get_contents('php://input'), true);

        if (!$this->isSecurityTagValid()) {
            status_header(400);
            exit('Invalid webhook signature.');
        }

        if (!$this->shouldProcessWebhook($data)) {
            status_header(200);
            exit('Webhook skipped');
        }

        $status = $data['transactionStatus'];

        try {
            $orderId = $this->payloadService->decode($data['payload']);
        } catch (TransactionCloudPayloadDecodeException $e) {
            status_header(400);
            exit($e->getMessage());
        }


        if (!$orderId) {
            status_header(400);
            exit('Invalid webhook payload.');
        }

        $order = wc_get_order($orderId);

        if (!$order) {
            status_header(404);
            exit('Order not found.');
        }

        if ($status == 'ONE_TIME_PAYMENT_STATUS_PAID') {
            $order->update_status('completed', 'Transaction Cloud payment completed.');
        }

        if ($status == 'TRANSACTION_STATUS_REFUNDED') {
            $order->update_status('refunded', 'Transaction Cloud refunded.');
        }

        status_header(200);

        exit('Webhook processed successfully.');
    }

    private function shouldProcessWebhook($data)
    {
        $requiredFields = [
            'payload',
            'transactionStatus',
            'webhookType'
        ];

        foreach ($requiredFields as $field) {
            if (!isset($data[$field])) {
                return false;
            }
        }

        $webhookType = $data['webhookType'];

        if ($webhookType != 'ONETIME_PAID' && $webhookType != 'ONETIME_REFUNDED') {
            return false;
        }

        return true;
    }

    /**
     * Checks whether webhook is sent by Transaction Cloud. If security tag
     * is not provideed in config returns true.
     *
     * @return bool
     */
    private function isSecurityTagValid()
    {
        $securityTag = $this->get_option('security_tag');

        if (!$securityTag) {
            return true;
        }

        $requestSecurityTag = TransacionCloudUtil::getRequestHeader('X-Transaction-Cloud-Security-Tag');

        if (!$requestSecurityTag) {
            return false;
        }

        return $securityTag == $requestSecurityTag;
    }
}
